import { ErrorHandler } from '../error/index.js';
export class AbstractPayloadValidation {
    isValidCommonParams(payload) {
        const commonParams = ['mode', 'orderId', 'amount'];
        const invalidParams = commonParams.filter((paramName) => {
            const value = payload[paramName];
            return !(paramName in payload) || (!value && value !== 0);
        });
        if (invalidParams.length) {
            throw ErrorHandler.invalidParameter(invalidParams);
        }
        if (typeof payload.amount !== 'number') {
            throw ErrorHandler.invalidParameterType('amount', 'number', typeof payload.amount);
        }
        return true;
    }
    isValidInstallments(payload) {
        if (!payload.installments || !payload.installmentType) {
            throw ErrorHandler.invalidParameter(['installments', 'installmentType']);
        }
        return true;
    }
    isValidCardInfo(card) {
        if (!card)
            throw ErrorHandler.emptyParam('card');
        if ('token' in card) {
            if (typeof card.token !== 'string') {
                throw ErrorHandler.invalidParameterType('card.token', 'string', typeof card.token);
            }
            if (!card.token) {
                throw ErrorHandler.emptyParam('card.token');
            }
        }
        if ('expiryDate' in card) {
            if (typeof card.expiryDate !== 'string') {
                throw ErrorHandler.invalidParameterType('card.expiryDate', 'string', typeof card.expiryDate);
            }
            if (!card.expiryDate.match(/^\d{4}$/i)) {
                throw ErrorHandler.invalidFormatParam('card.expiryDate', 'mmyy');
            }
        }
        if ('number' in card) {
            if (typeof card.number !== 'string') {
                throw ErrorHandler.invalidParameterType('card.number', 'string', typeof card.number);
            }
            if (!card.number.match(/^\d{16}$/i)) {
                throw ErrorHandler.invalidFormatParam('card.number', '16 digits (only numbers)');
            }
        }
        if (!card.number && !card.token) {
            throw ErrorHandler.parameterNotFound(['card.number', 'card.token']);
        }
        return true;
    }
    credit(payload) {
        this.isValidCardInfo(payload.card);
        this.isValidInstallments(payload);
        if ('postponeConfirmation' in payload &&
            typeof payload.postponeConfirmation !== 'boolean') {
            throw ErrorHandler.invalidParameterType('postponeConfirmation', 'boolean', typeof payload.postponeConfirmation);
        }
        return true;
    }
    confirm(payload) {
        if (!('nit' in payload)) {
            throw ErrorHandler.parameterNotFound('nit');
        }
        if (typeof payload.nit !== 'string') {
            throw ErrorHandler.invalidParameterType('nit', 'string', typeof payload.nit);
        }
        return true;
    }
    storeCard(payload) {
        this.isValidCardInfo(payload.card);
        if (!('customerId' in payload)) {
            throw ErrorHandler.parameterNotFound('customerId');
        }
        if (!payload.customerId) {
            throw ErrorHandler.emptyParam('customerId');
        }
        if (typeof payload.customerId !== 'string') {
            throw ErrorHandler.invalidParameterType('customerId', 'string', typeof payload.customerId);
        }
        if (payload.customerId.length > 20) {
            throw ErrorHandler.invalidFormatParam('customerId', 'max. 20 characters');
        }
        return true;
    }
    verifyCard(payload) {
        this.isValidCardInfo(payload.card);
        return true;
    }
    pix(payload) {
        this.isValidInstallments(payload);
        return true;
    }
    billet(payload) {
        return true;
    }
    debit(payload) {
        return true;
    }
    getModalityFunction(modality) {
        const functionToCall = this[modality] || null;
        if (typeof functionToCall !== 'function') {
            throw ErrorHandler.invalidChoice('modality', modality);
        }
        return functionToCall;
    }
    isValidPayload(payload) {
        this.isValidCommonParams(payload);
        const functionToCall = this.getModalityFunction(payload.mode);
        functionToCall.apply(this, [payload]);
        return true;
    }
}
