import fs from 'fs';
import { Logger } from './Logger.js';
import ms from 'ms';
import path from 'path';
class LogCleanup {
    async getLogFiles() {
        return new Promise((resolve, reject) => {
            const { API_LOG_PATH } = process.env;
            if (!API_LOG_PATH) {
                Logger.error('Fail execute log cleanup. Invalid log path.');
                return reject('Invalid log path');
            }
            fs.readdir(API_LOG_PATH, (err, files) => {
                if (err)
                    return reject(err);
                resolve(files);
            });
        });
    }
    async getStatFile(file) {
        const fileName = path.basename(file);
        if (fileName.startsWith('.')) {
            return Promise.reject(new Error('Is hidden file'));
        }
        return new Promise((resolve, reject) => {
            fs.stat(file, (err, stats) => {
                if (err) {
                    Logger.error(`Error in fs.stats for file ${file}. ${err.message}`);
                    return reject(err);
                }
                resolve({ file, stats });
            });
        });
    }
    async filterOldFiles(files, maxAgeMs) {
        const oldFiles = [];
        const { API_LOG_PATH } = process.env;
        if (!API_LOG_PATH)
            return oldFiles;
        const statsPromise = files.map((file) => {
            const filePath = path.join(API_LOG_PATH, file);
            return this.getStatFile(filePath);
        });
        const now = new Date().getTime();
        const results = await Promise.allSettled(statsPromise);
        results.forEach((result) => {
            if (result.status === 'fulfilled') {
                const { file, stats } = result.value;
                if (stats.isFile() && (now - stats.birthtime.getTime()) > maxAgeMs) {
                    console.log(stats.birthtime.getTime(), maxAgeMs);
                    oldFiles.push(file);
                }
            }
        });
        return oldFiles;
    }
    async removeFile(file) {
        return new Promise((resolve, reject) => {
            fs.unlink(file, (err) => {
                if (err) {
                    Logger.error('Fail cleanup log', { file, error: err.message });
                    return reject(err);
                }
                resolve({ file });
            });
        });
    }
    async removeFiles(files) {
        const removeFilesPromise = files.map(this.removeFile);
        return Promise.allSettled(removeFilesPromise);
    }
    async start() {
        const removedFiles = [];
        try {
            const { API_MAX_DAYS_KEEP_LOG = '90d' } = process.env;
            const maxAgeMs = ms(API_MAX_DAYS_KEEP_LOG);
            const files = await this.getLogFiles();
            const filesToRemove = await this.filterOldFiles(files, maxAgeMs);
            const results = await this.removeFiles(filesToRemove);
            results.forEach(result => {
                if (result.status === 'fulfilled') {
                    removedFiles.push(result.value.file);
                }
                ;
            });
            return {
                removedFiles,
            };
        }
        catch (err) {
            Logger.error('Fail to execute log cleanup.', { error: err.message });
            return { removedFiles };
        }
    }
}
const instance = new LogCleanup();
export { instance as LogCleanup };
